@@ -7,17 +7,23 @@
 # -*- coding: utf-8 -*-
 
 import os
+from math import cos, sqrt, pi
+from haversine import haversine
 import numpy as np
 import pandas as pd
 from os import path
+import dill as pickle
 from compress_pickle import dump, load
 from vincenty import vincenty
 
 
+ptB = [145.72726054, -16.81220888]
+ptA = [145.7274588, -16.81253358]
+
 ###############################################################################
 # Vincenty distance between points
 ###############################################################################
-def vincentyDistance(pointA, pointB, meters=False):
+def vincentyDistance(pointA, pointB, meters=True):
     """Calculates the Vincenty arc distance between points.
     
     Args:
@@ -31,12 +37,73 @@ def vincentyDistance(pointA, pointB, meters=False):
         (pointA[1], pointA[0]), 
         (pointB[1], pointB[0])
     )
+    if distKM == None:
+        return 0
     if meters:
         return distKM*1000
     else:
         return distKM
 
 
+def cheapRuler(pointA, pointB):
+    """Calculates the distance between two (lon,lat) points assumming flat approximations.
+        https://blog.mapbox.com/fast-geodesic-approximations-with-cheap-ruler-106f229ad016
+    """
+
+    """
+    (parLen, merLen) = (40074.275e3, 20004.146e3)
+    (lonA, latA) = pointA
+    (lonB, latB) = pointB
+    # Distance calculations
+    (dx, dy) = (
+        parLen*(abs(lonA-lonB)/360)*cos((latA+latB)/2),
+        merLen*(abs(latA-latB)/180)
+    )
+    distance = sqrt(dx**2+dy**2)
+    """
+
+    """Following code based on:
+        https://github.com/mapbox/cheap-ruler/blob/master/index.js
+    """
+    (lonA, latA) = pointA
+    (lonB, latB) = pointB
+
+    RE = 6378.137
+    FE = 1 / 298.257223563
+    E2 = FE * (2 - FE)
+    RAD = pi / 180
+
+    m = RAD * RE * 1000
+    coslat = cos(latA * RAD)
+    w2 = 1 / (1- E2 * (1 - coslat * coslat))
+    w = sqrt(w2)
+
+    kx = m * w * coslat
+    ky = m * w * w2 * (1 - E2)
+
+    deg = lonA - lonB
+
+    while (deg < -180): 
+        deg += 360
+    while (deg > 180): 
+        deg -= 360
+
+    dx = deg * kx
+    dy = (latA - latB) * ky
+    distance = sqrt(dx**2+dy**2)
+
+    return distance
+
+
+def haversineDistance(pointA, pointB):
+    """Calculates the distance between two (lon,lat) points using the haversine approximation.
+
+    """
+    (lonA, latA) = pointA
+    (lonB, latB) = pointB
+    distance = haversine((latB, lonB), (latA, lonA), unit='m')
+    return distance
+
 ###############################################################################
 # Various auxiliary functions
 ###############################################################################
@@ -93,10 +160,15 @@ def dumpLandscape(landscape, fPath, fName, fExt='bz2'):
         fName (string): Filename.
         fExt (string): File extension.
     """
-    dump(
-        landscape, 
-        path.join(fPath, '{}.{}'.format(fName, fExt))
-    )
+    if fExt == 'bz2':
+        dump(
+            landscape, 
+            path.join(fPath, '{}.{}'.format(fName, fExt))
+        )
+    else:
+        with open(path.join(fPath, '{}.{}'.format(fName, fExt)), 'wb') as f:
+            pickle.dump(landscape, f)
+        
 
 def loadLandscape(fPath, fName, fExt='bz2'):
     """Loads a serialized landscape from disk.
@@ -109,9 +181,13 @@ def loadLandscape(fPath, fName, fExt='bz2'):
     Returns:
         (object): Landscape object.
     """
-    lnd = load(
-        path.join(fPath, '{}.{}'.format(fName, fExt))
-    )
+    if fExt == 'bz2':
+        lnd = load(
+            path.join(fPath, '{}.{}'.format(fName, fExt))
+        )
+    else:
+        with open(path.join(fPath, '{}.{}'.format(fName, fExt)), 'rb') as f:
+            lnd = pickle.load(f)
     return lnd
 
 def exportLandscape(landscape, fPath, fName):
