@@ -12,6 +12,7 @@ from scipy.optimize import fsolve
 from sklearn.preprocessing import normalize
 import MGSurvE.constants as cst
 import warnings
+import matplotlib.pyplot as plt
 warnings.filterwarnings('ignore', 'The iteration is not making good progress')
 
 # https://en.wikipedia.org/wiki/Sigmoid_function
@@ -101,9 +102,17 @@ def zeroInflatedExponentialKernel(
                 migrMat[i][j] = 0
             else:
                 migrMat[i][j] = truncatedExponential(dst, params=params)
-        migrMat[i] = migrMat[i] / np.sum(migrMat[i]) * (1 - zeroInflation)
+        # migrMat[i] = migrMat[i] / np.sum(migrMat[i]) * (1 - zeroInflation)
+        migrRowSum = np.sum(migrMat[i])
+        for j in range(len(row)):
+            migrMat[i][j] = migrMat[i][j] / migrRowSum * (1 - zeroInflation)
+            if np.isnan(migrMat[i][j]):
+                # print("NaN Warning (check points locations, distances might be too large.")
+                migrMat[i][j] = 0
+
     np.fill_diagonal(migrMat, zeroInflation)
     tauN = normalize(migrMat, axis=1, norm='l1')
+    
     return tauN
 
 
@@ -174,6 +183,7 @@ def nSolveKernel(kernelDict, yVal, guess=0, latlon=False, R=6371):
     Returns:
         float: Distance for the probability value.
     '''
+    # https://stackoverflow.com/questions/5644836/in-python-how-does-one-catch-warnings-as-if-they-were-exceptions
     (kFun, kPar) = (kernelDict['kernel'], kernelDict['params'])
     func = lambda delta : yVal-kFun(delta, **kPar)
     distance = fsolve(func, guess)
@@ -181,4 +191,8 @@ def nSolveKernel(kernelDict, yVal, guess=0, latlon=False, R=6371):
         dist = math.atan(distance[0]/R)
     else:
         dist = distance[0]
-    return dist
\ No newline at end of file
+    # Negative radius patch ---------------------------------------------------
+    if dist < 0:
+        return 0
+    else:
+        return dist
\ No newline at end of file
