@@ -7,8 +7,9 @@
 
 import math
 import numpy as np
+import pandas as pd
 from time import time
-import vincenty as vin
+from haversine import haversine
 from sklearn.preprocessing import normalize
 from sklearn.cluster import KMeans
 import MGSurvE.matrices as mat
@@ -30,6 +31,7 @@ class Landscape:
         kernelFunction (function): Function that determines de relationship between distances and migration probabilities.
         kernelParams (dict): Parameters required for the kernel function to determine migration probabilities.
         maskingMatrix (numpy array): Matrix that determines the probability of shifting from one point-type to another one (squared with size equal to the number of point types). If None, every point-type transition is equiprobable.
+        attractionVector (list): Relative attraction of one site to another (does not preserve migration matrix diagonal)
 
         distanceMatrix (numpy array): Matrix with the distances between all the points in the landscape. If None, it's auto-calculated (see calcPointsDistances).
         migrationMatrix (numpy array): Markov matrix that determines the probability of moving from one site to another. If None, it's auto-calculated (see calcPointsMigration).
@@ -48,7 +50,8 @@ class Landscape:
     # Initializers
     ###########################################################################
     def __init__(self, 
-        points, 
+        points,
+        pointsTrapBanned=None,
         maskingMatrix=None,
         attractionVector=None,
 
@@ -96,6 +99,7 @@ class Landscape:
         self.populations = populations
         self.latlon = False
         self.landLimits = landLimits
+        self.reversePlot = False
         # Check and define coordinates ----------------------------------------
         ptsHead = set(points.columns)
         if ('x' in ptsHead) and ('y' in ptsHead):
@@ -107,8 +111,11 @@ class Landscape:
             self.geometryType = 'll'
             self.pointCoords = np.asarray(points[['lon', 'lat']])
             self.latlon = True
+            self.reversePlot = True
             if distanceFunction is None:
-                self.distanceFunction = aux.vincentyDistance
+                self.distanceFunction = lambda a, b: haversine(
+                    (a[1], a[0]), (b[1], b[0]), unit='m'
+                )
         else:
             raise Exception(
                 '''Check the landscape type! 
@@ -120,6 +127,11 @@ class Landscape:
             self.pointTypes = np.asarray(points['t'])
         else:
             self.pointTypes = np.asarray([0]*len(points))
+        # Check and define point-IDs ------------------------------------------
+        if ('id' in ptsHead):
+            self.pointID = tuple(points['id'])
+        else:
+            self.pointID = tuple(range(len(points)))
         # If no migration mask is provided, generate a dummy one --------------
         if maskingMatrix is None:
             ptNum = len(set(self.pointTypes))
@@ -128,7 +140,7 @@ class Landscape:
             self.maskingMatrix = np.asarray(maskingMatrix)
         # If no attraction vector is provided, generate a dummy one -----------
         if attractionVector is None:
-            ptNum = len(set(self.pointTypes))
+            ptNum = self.pointNumber
             self.attractionVector = np.full(ptNum, 1)
         else:
             self.attractionVector = np.asarray(attractionVector)
@@ -161,12 +173,16 @@ class Landscape:
         else:
             self.landLimits = landLimits
         # Init traps locations ------------------------------------------------
+        if pointsTrapBanned:
+            self.pointsTrapBanned = set(pointsTrapBanned)
+        else:
+            self.pointsTrapBanned = pointsTrapBanned
         if (traps is not None):
             tpsHead = set(traps.columns)
             if (self.geometryType == 'xy'):
-                self.trapsCoords = np.asarray(traps[['x', 'y']])
+                self.trapsCoords = np.asarray(traps[['x', 'y']]).astype(float)
             else:
-                self.trapsCoords = np.asarray(traps[['lon', 'lat']])
+                self.trapsCoords = np.asarray(traps[['lon', 'lat']]).astype(float)
             # Check if there's trap-type information --------------------------
             if ('t' in tpsHead):
                 self.trapsTypes = np.asarray(traps['t'])
@@ -176,16 +192,36 @@ class Landscape:
                 self.trapsFixed = np.asarray(traps['f'])
             else:
                 self.trapsFixed = np.asarray([0]*len(self.trapsCoords))
+            if ('sid' in tpsHead):
+                self.trapsSiteID = list(traps['sid'])
+            if ('o' in tpsHead):
+                self.trapsTOptim = np.asarray(traps['o'])
+            else:
+                self.trapsTOptim = np.asarray([0]*len(self.trapsCoords))
             self.trapsNumber = len(self.trapsCoords)
             # Init trap types -------------------------------------------------
+            trapTypesNum = len(set(self.trapsTypes))
+            pointTypesNum = len(set(self.pointTypes))
             if trapsMask is None:
                 mskShape = (
-                    len(set(self.trapsTypes)), 
+                    max(set(self.trapsTypes))+1, 
                     len(set(self.pointTypes))
                 )
                 self.trapsMask = np.full(mskShape, 1)
             else:
                 self.trapsMask = trapsMask
+                if (self.trapsMask.shape[0] != trapTypesNum):
+                    raise Exception(
+                        '''Number of trap types in mask ({}) is not the same as 
+                        trap types in traps dictionary ({}).
+                        '''.format(self.trapsMask.shape[0], trapTypesNum)
+                    )
+                if (self.trapsMask.shape[1] != pointTypesNum):
+                    raise Exception(
+                        '''Number of point types in mask ({}) is not the same as 
+                        the ones defined in the landscape ({}).
+                        '''.format(self.trapsMask.shape[1], pointTypesNum)
+                    )
             # Calculate trapsDistances ----------------------------------------
             self.calcTrapsDistances()
             # Generate empty traps matrix -------------------------------------
@@ -209,10 +245,12 @@ class Landscape:
     def calcPointsMigration(self):
         """Calculates the migrationMatrix amongst the points (in place).
         """
-        self.migrationMatrix = self.kernelFunction(
+        preAttractMigMatrix = self.kernelFunction(
             self.distanceMatrix, **self.kernelParams
         )
-
+        self.migrationMatrix = mat.calcAttractiveness(
+            preAttractMigMatrix, self.attractionVector
+        )
     def calcPointsMaskedMigration(self):
         """Calculates the maskedMigrationMatrix depending on point-type (in place).
         """
@@ -236,7 +274,8 @@ class Landscape:
             self.trapsKernels, self.trapsMask, self.pointTypes
         )
         ptsN = self.pointNumber
-        self.trapsMigration[:ptsN, ptsN:] = trapProbs
+        self.trapsMigration[:ptsN, :ptsN] = np.copy(self.maskedMigration)
+        self.trapsMigration[:ptsN, ptsN:] = np.copy(trapProbs)
         self.trapsMigration = normalize(self.trapsMigration, axis=1, norm='l1')
     def updateTrapsCoords(self, trapsCoords):
         self.trapsCoords = trapsCoords
@@ -259,6 +298,11 @@ class Landscape:
         self.trapsNumber = len(self.trapsCoords)
         self.trapsKernels = trapsKernels
         self.trapsFixed = traps['f']
+        try:
+            self.trapsTOptim = traps['o']
+        except:
+            # print("No 'o' part of update")
+            pass
         # Updating necessary matrices -----------------------------------------
         if (oldTrapsNum != len(self.trapsCoords)):
             self.trapsMigration = mat.genVoidFullMigrationMatrix(
@@ -283,7 +327,7 @@ class Landscape:
             })
         self.trapsKernels = tker
     ###########################################################################
-    # Fitness Function
+    # Optimization
     ###########################################################################
     def calcFundamentalMatrix(self):
         """Calculates the Markov fundamental matrix on the landscape.
@@ -331,6 +375,9 @@ class Landscape:
         ):
         """Plots the sites coordinates.
         """
+        # pCoords = self.pointCoords
+        # if reversePlot:
+        #     pCoords = np.asarray([[i[1], i[0]] for i in self.pointCoords])
         (fig, ax) = plt.plotSites(
             fig, ax, 
             self.pointCoords, self.pointTypes,
@@ -434,3 +481,13 @@ class Landscape:
         (maxX, maxY) = np.apply_along_axis(max, 0, self.pointCoords)
         retPair = ((minX, maxX), (minY, maxY))
         return retPair
+    ###########################################################################
+    # Export Landscape
+    ###########################################################################
+    def exportForMGDrivE(self):
+        (xyp, xyt) = (self.pointCoords, self.trapsCoords)
+        (idp, idt) = (self.pointTypes, (self.trapsTypes*-1)-1)
+        xyPos = np.vstack([xyp, xyt])
+        idTyp = np.hstack([idp, idt])
+        df = pd.DataFrame({'x': xyPos.T[0], 'y': xyPos.T[1], 'type': idTyp})
+        return {'pos': df, 'mig': self.trapsMigration}
\ No newline at end of file
