@@ -1,8 +1,9 @@
-'''Data-Visualization functions.
+'''Data-Visualization functions (thanks to Elijah Bartolome in the impelemntation of some of the visualization functions).
 
 '''
 
-
+import warnings
+import matplotlib
 from os import path
 from math import log
 import matplotlib.pyplot as plt
@@ -10,8 +11,6 @@ import MGSurvE.constants as cst
 import networkx as nx
 from sklearn.preprocessing import normalize
 import numpy as np
-import cartopy.crs as ccrs
-import cartopy.feature as cfeature
 
 
 def plotSites(
@@ -186,27 +185,6 @@ def plotTrapsNetwork(
     return (fig, ax)
 
 
-def plotLandBoundary(fig, ax, landTuples=cst.LAND_TUPLES):
-    """ Plots the land's boundary as a polygon.
-
-    Parameters:
-        fig (matplotlib): Matplotlib fig object.
-        ax (matplotlib): Matplotlib ax object.
-        landTuples (list of tuples): Check the constants.py file for format.
-    
-    Returns:
-        (fig, ax): Matplotlib (fig, ax) tuple.
-    """ 
-    lands = [
-        cfeature.NaturalEarthFeature(
-            'physical', 'land', i[0],
-            edgecolor=i[1], facecolor='#00000000', linewidth=i[2]
-        ) for i in landTuples
-    ]
-    [ax.add_feature(i, zorder=(-50+ix)) for (ix, i) in enumerate(lands)]
-    return (fig, ax)
-
-
 def plotMatrix(
         fig, ax,
         matrix, 
@@ -249,7 +227,7 @@ def plotMatrix(
     return (fig, ax)
 
 
-def plotClean(fig, ax, frame=False, bbox=None, labels=False):
+def plotClean(fig, ax, frame=False, bbox=None, labels=False, pad=(0, 0)):
     """ Makes axes equally spaced and removes frame.
 
     Parameters:
@@ -260,6 +238,11 @@ def plotClean(fig, ax, frame=False, bbox=None, labels=False):
     Returns:
         (fig, ax): Matplotlib (fig, ax) tuple.
     """ 
+    if bbox is not None:
+        bbox = (
+            (bbox[0][0]-pad[0], bbox[0][1]+pad[0]), 
+            (bbox[1][0]-pad[1], bbox[1][1]+pad[1])
+        )
     ax.set_aspect('equal')
     if frame is not True:
         ax.axis('off')
@@ -473,4 +456,62 @@ def saveFig(
         pad_inches=pad_inches, transparent=transparent, 
         **kwargs
     )
-    return (fig, ax)
\ No newline at end of file
+    return (fig, ax)
+
+
+def plotTrapsKernels(
+        fig, ax, lnd,
+        colors=cst.TRP_COLS, maxSca=5, alpha=.75,
+        distRange=(0, 100), aspect=.3
+    ):
+    kers = lnd.trapsKernels
+    # dMax = max(max([kers[i]['radii'] for i in range(len(kers))])) * maxSca
+    dMax = distRange[1]
+    # Generate figure
+    for i in range(len(kers)):
+        ker = kers[i]
+        dists = np.arange(0, dMax+1, dMax/100)
+        probs = [ker['kernel'](d, **ker['params']) for d in dists]
+        ax.plot(dists, probs, color=colors[i], lw=4, alpha=alpha)
+    ax.set_xlim(0, dMax)
+    ax.set_ylim(0, 1)
+    ax.set_aspect(aspect/ax.get_data_ratio())
+    return (fig, ax)
+
+
+def plotsClearMemory():
+    # https://stackoverflow.com/questions/28757348/how-to-clear-memory-completely-of-all-matplotlib-plots
+    allfignums = matplotlib.pyplot.get_fignums()
+    for i in allfignums:
+        fig = matplotlib.pyplot.figure(i)
+        fig.clear()
+        matplotlib.pyplot.close(fig)
+
+
+###############################################################################
+# Cartopy dependency
+###############################################################################
+try:
+    import cartopy.feature as cfeature
+except ImportError:
+    warnings.warn("cartopy installation was not detected! Geo-boundaries (plotLandBoundary) not available!")
+else:
+    def plotLandBoundary(fig, ax, landTuples=cst.LAND_TUPLES):
+        """ Plots the land's boundary as a polygon.
+
+        Parameters:
+            fig (matplotlib): Matplotlib fig object.
+            ax (matplotlib): Matplotlib ax object.
+            landTuples (list of tuples): Check the constants.py file for format.
+        
+        Returns:
+            (fig, ax): Matplotlib (fig, ax) tuple.
+        """ 
+        lands = [
+            cfeature.NaturalEarthFeature(
+                'physical', 'land', i[0],
+                edgecolor=i[1], facecolor='#00000000', linewidth=i[2]
+            ) for i in landTuples
+        ]
+        [ax.add_feature(i, zorder=(-50+ix)) for (ix, i) in enumerate(lands)]
+        return (fig, ax)
